==> server-core-0.0.1.jar: aclaimant/server/middleware.clj <==
(ns aclaimant.server.middleware
  (:require
    [aclaimant.server.core.request :as req]
    [cheshire.core :as json]
    [clojure.walk :as walk]
    [outpace.config :refer [defconfig]]
    [ring.middleware.json :as json-middleware]
    [ring.middleware.reload :as reload]))

(defconfig reload-middleware-enabled? false)

(defn wrap-allow-cross-origin
  "middleware function to allow crosss origin - TODO restrict further"
  [handler]
  (fn [request]
    (let [response (handler request)]
      (assoc-in
        (assoc-in response [:headers "Access-Control-Allow-Origin"] "*")
        [:headers "Access-Control-Allow-Methods"] "*"))))

(defn wrap-json-errors
  ""
  [handler]
  (fn [request]
    (try
      (handler request)
      (catch Throwable e
        (println "[ERROR] an error occurred while processing a request."
                 (.getMessage e)
                 "\n"
                 (with-out-str (.printStackTrace e)))
        {:status 500
         :headers {"Content-Type" "application/json"}
         :body (json/generate-string
                 {:error (str (.getClass e))
                  :message (.getMessage e)})}))))

(defn- ->json-keys [s]
  (-> s
      json/parse-string
      req/jsonify-keys
      json/generate-string))

(defn wrap-json-request-params
  "Converts the snake-case keys in :params to dashified keys"
  [handler]
  (fn [req]
    (let [orig-params (:params req)
          orig-json-params (:json-params req)
          params (req/dashify-keys orig-params)
          json-params (req/dashify-keys orig-json-params)]
      (handler
        (assoc req
               :params (merge params json-params)
               :original-params orig-params
               :json-params json-params
               :original-json-params orig-json-params)))))

(defn wrap-json-response
  "Fixes all the nastiness of the dashes and nonsense"
  [handler]
  (fn [request]
    (let [{:keys [headers] :as response} (handler request)]
      (if (= (get headers "Content-Type") "application/json")
        (update-in response [:body] ->json-keys)
        response))))

(defn wrap-logged [handler]
  (fn [{:keys [uri headers] :as request}]
    (let [start-time (System/currentTimeMillis)
          response (handler request)
          elapsed-millis (- (System/currentTimeMillis) start-time)]
      (println (format "[REQUEST] response=%s uri=%s response-millis=%s inbound-headers=%s"
                       (get-in response [:status])
                       uri
                       elapsed-millis
                       headers))
      response)))


(defn api-common
  ""
  [routes]
  (-> routes
      wrap-allow-cross-origin
      wrap-json-response
      wrap-json-request-params
      (json-middleware/wrap-json-body {:keywords? true :bigdecimals? true})
      json-middleware/wrap-json-params
      wrap-json-errors))

(defn older-api-common
  ""
  [routes]
  (-> routes
      wrap-allow-cross-origin
      (json-middleware/wrap-json-body {:keywords? true :bigdecimals? true})
      json-middleware/wrap-json-params
      wrap-json-errors))

(defn _id->id [response]
  (walk/postwalk
    (fn [m]
      (if (and (not (coll? m))
               (= (keyword m) :_id))
        :id
        m))
    response))

(defn ^:private body-fix-ids [body]
  (-> body
      json/parse-string
      _id->id
      json/generate-string))

(defn wrap-change-ids
  "Middleware that converts all :_id to :id
  in your response."
  [handler]
  (fn [request]
    (let [{:keys [headers] :as response} (handler request)]
      (if (= (get headers "Content-Type") "application/json")
        (update-in response [:body] body-fix-ids)
        response))))

(defn wrap-maybe-reload
  "Wrap with ring reload if configured using
  `{aclaimant.middleware.core/reload-middleware-enabled? true}`
  in your config.edn. Defaults to not reloading."
  [handler]
  (println "[Middleware] Wrap reload is set to" reload-middleware-enabled?)
  (if reload-middleware-enabled?
    (reload/wrap-reload handler)
    handler))

==> server-core-0.0.3-20150819.205606-1.jar: aclaimant/server/middleware.clj <==
(ns aclaimant.server.middleware
  (:require
    [aclaimant.server.core.request :as req]
    [cheshire.core :as json]
    [clojure.walk :as walk]
    [outpace.config :refer [defconfig]]
    [ring.middleware.json :as json-middleware]
    [ring.middleware.reload :as reload]))

(defconfig reload-middleware-enabled? false)

(defn wrap-allow-cross-origin
  "middleware function to allow crosss origin - TODO restrict further"
  [handler]
  (fn [request]
    (let [response (handler request)]
      (assoc-in
        (assoc-in response [:headers "Access-Control-Allow-Origin"] "*")
        [:headers "Access-Control-Allow-Methods"] "*"))))

(defn wrap-json-errors
  ""
  [handler]
  (fn [request]
    (try
      (handler request)
      (catch Throwable e
        (println "[ERROR] an error occurred while processing a request."
                 (.getMessage e)
                 "\n"
                 (with-out-str (.printStackTrace e)))
        {:status 500
         :headers {"Content-Type" "application/json"}
         :body (json/generate-string
                 {:error (str (.getClass e))
                  :message (.getMessage e)})}))))

(defn- ->json-keys [s]
  (-> s
      json/parse-string
      req/jsonify-keys
      json/generate-string))

(defn wrap-json-request-params
  "Converts the snake-case keys in :params to dashified keys"
  [handler]
  (fn [req]
    (let [orig-params (:params req)
          orig-json-params (:json-params req)
          params (req/dashify-keys orig-params)
          json-params (req/dashify-keys orig-json-params)]
      (handler
        (assoc req
               :params (merge params json-params)
               :original-params orig-params
               :json-params json-params
               :original-json-params orig-json-params)))))

(defn wrap-json-response
  "Fixes all the nastiness of the dashes and nonsense"
  [handler]
  (fn [request]
    (let [{:keys [headers] :as response} (handler request)]
      (if (= (get headers "Content-Type") "application/json")
        (update-in response [:body] ->json-keys)
        response))))

(defn wrap-logged [handler]
  (fn [{:keys [uri headers] :as request}]
    (let [start-time (System/currentTimeMillis)
          response (handler request)
          elapsed-millis (- (System/currentTimeMillis) start-time)]
      (println (format "[REQUEST] response=%s uri=%s response-millis=%s inbound-headers=%s"
                       (get-in response [:status])
                       uri
                       elapsed-millis
                       headers))
      response)))


(defn api-common
  ""
  [routes]
  (-> routes
      wrap-allow-cross-origin
      wrap-json-response
      wrap-json-request-params
      (json-middleware/wrap-json-body {:keywords? true :bigdecimals? true})
      json-middleware/wrap-json-params
      wrap-json-errors))

(defn older-api-common
  ""
  [routes]
  (-> routes
      wrap-allow-cross-origin
      (json-middleware/wrap-json-body {:keywords? true :bigdecimals? true})
      json-middleware/wrap-json-params
      wrap-json-errors))

(defn _id->id [response]
  (walk/postwalk
    (fn [m]
      (if (and (not (coll? m))
               (= (keyword m) :_id))
        :id
        m))
    response))

(defn ^:private body-fix-ids [body]
  (-> body
      json/parse-string
      _id->id
      json/generate-string))

(defn wrap-change-ids
  "Middleware that converts all :_id to :id
  in your response."
  [handler]
  (fn [request]
    (let [{:keys [headers] :as response} (handler request)]
      (if (= (get headers "Content-Type") "application/json")
        (update-in response [:body] body-fix-ids)
        response))))

(defn wrap-maybe-reload
  "Wrap with ring reload if configured using
  `{aclaimant.server.middleware.core/reload-middleware-enabled? true}`
  in your config.edn. Defaults to not reloading."
  [handler]
  (println "[Middleware] Wrap reload is set to" reload-middleware-enabled?)
  (if reload-middleware-enabled?
    (reload/wrap-reload handler)
    handler))
